(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.ternaryPlot = {}));
}(this, function (exports) { 'use strict';

  var camelToDashMap = new Map();
  function camelToDash(str) {
    var result = camelToDashMap.get(str);

    if (result === undefined) {
      result = str.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
      camelToDashMap.set(str, result);
    }

    return result;
  }
  function pascalToDash(str) {
    return camelToDash(str.replace(/((?!([A-Z]{2}|^))[A-Z])/g, '-$1'));
  }
  function shadyCSS(fn, fallback) {
    var shady = window.ShadyCSS;
    /* istanbul ignore next */

    if (shady && !shady.nativeShadow) {
      return fn(shady);
    }

    return fallback;
  }
  function stringifyElement(element) {
    var tagName = String(element.tagName).toLowerCase();
    return "<".concat(tagName, ">");
  }
  var IS_IE = 'ActiveXObject' in window;

  function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

  var defaultTransform = function defaultTransform(v) {
    return v;
  };

  var objectTransform = function objectTransform(value) {
    if (_typeof(value) !== 'object') {
      throw TypeError("Assigned value must be an object: ".concat(typeof v === "undefined" ? "undefined" : _typeof(v)));
    }

    return value && Object.freeze(value);
  };

  function property(value, connect) {
    var type = _typeof(value);

    var transform = defaultTransform;

    switch (type) {
      case 'string':
        transform = String;
        break;

      case 'number':
        transform = Number;
        break;

      case 'boolean':
        transform = Boolean;
        break;

      case 'function':
        transform = value;
        value = transform();
        break;

      case 'object':
        if (value) Object.freeze(value);
        transform = objectTransform;
        break;

      default:
        break;
    }

    return {
      get: function get(host) {
        var val = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : value;
        return val;
      },
      set: function set(host, val, oldValue) {
        return transform(val, oldValue);
      },
      connect: type !== 'object' && type !== 'undefined' ? function (host, key, invalidate) {
        if (host[key] === value) {
          var attrName = camelToDash(key);

          if (host.hasAttribute(attrName)) {
            var attrValue = host.getAttribute(attrName);
            host[key] = attrValue !== '' ? attrValue : true;
          }
        }

        return connect && connect(host, key, invalidate);
      } : connect
    };
  }

  function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

  function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

  function _typeof$1(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$1 = function _typeof(obj) { return typeof obj; }; } else { _typeof$1 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$1(obj); }

  function render(_get) {
    var customOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (typeof _get !== 'function') {
      throw TypeError("The first argument must be a function: ".concat(_typeof$1(_get)));
    }

    var options = _objectSpread({
      shadowRoot: true
    }, customOptions);

    var shadowRootInit = {
      mode: 'open'
    };

    if (_typeof$1(options.shadowRoot) === 'object') {
      Object.assign(shadowRootInit, options.shadowRoot);
    }

    return {
      get: function get(host) {
        var fn = _get(host);

        return function flush() {
          fn(host, options.shadowRoot ? host.shadowRoot : host);
        };
      },
      connect: function connect(host) {
        if (options.shadowRoot && !host.shadowRoot) {
          host.attachShadow(shadowRootInit);
        }
      },
      observe: function observe(host, fn) {
        fn();
      }
    };
  }

  var targets = new WeakMap();

  function getListeners(target) {
    var listeners = targets.get(target);

    if (!listeners) {
      listeners = new Set();
      targets.set(target, listeners);
    }

    return listeners;
  }

  var queue = new Set();

  var run = function run(fn) {
    return fn();
  };

  function execute() {
    try {
      queue.forEach(function (target) {
        try {
          getListeners(target).forEach(run);
          queue.delete(target);
        } catch (e) {
          queue.delete(target);
          throw e;
        }
      });
    } catch (e) {
      if (queue.size) execute();
      throw e;
    }
  }

  function dispatch(target) {
    if (!queue.size) {
      requestAnimationFrame(execute);
    }

    queue.add(target);
  }
  function subscribe(target, cb) {
    var listeners = getListeners(target);
    listeners.add(cb);
    dispatch(target);
    return function () {
      return listeners.delete(cb);
    };
  }

  var entries = new WeakMap();
  function getEntry(target, key) {
    var targetMap = entries.get(target);

    if (!targetMap) {
      targetMap = new Map();
      entries.set(target, targetMap);
    }

    var entry = targetMap.get(key);

    if (!entry) {
      entry = {
        target: target,
        key: key,
        value: undefined,
        contexts: undefined,
        deps: undefined,
        state: 1,
        checksum: 0,
        observed: false
      };
      targetMap.set(key, entry);
    }

    return entry;
  }

  function calculateChecksum(entry) {
    var checksum = entry.state;

    if (entry.deps) {
      entry.deps.forEach(function (depEntry) {
        // eslint-disable-next-line no-unused-expressions
        depEntry.target[depEntry.key];
        checksum += depEntry.state;
      });
    }

    return checksum;
  }

  function dispatchDeep(entry) {
    if (entry.observed) dispatch(entry);
    if (entry.contexts) entry.contexts.forEach(dispatchDeep);
  }

  var context = null;
  function get(target, key, getter) {
    var entry = getEntry(target, key);

    if (context === entry) {
      context = null;
      throw Error("Circular '".concat(key, "' get invocation in '").concat(stringifyElement(target), "'"));
    }

    if (context) {
      context.deps = context.deps || new Set();
      context.deps.add(entry);
    }

    if (context && (context.observed || context.contexts && context.contexts.size)) {
      entry.contexts = entry.contexts || new Set();
      entry.contexts.add(context);
    }

    var parentContext = context;
    context = entry;

    if (entry.checksum && entry.checksum === calculateChecksum(entry)) {
      context = parentContext;
      return entry.value;
    }

    if (entry.deps && entry.deps.size) {
      entry.deps.forEach(function (depEntry) {
        if (depEntry.contexts) depEntry.contexts.delete(entry);
      });
      entry.deps = undefined;
    }

    try {
      var nextValue = getter(target, entry.value);

      if (nextValue !== entry.value) {
        entry.state += 1;
        entry.value = nextValue;
        dispatchDeep(entry);
      }

      entry.checksum = calculateChecksum(entry);
      context = parentContext;
    } catch (e) {
      context = null;
      throw e;
    }

    return entry.value;
  }
  function set(target, key, setter, value) {
    if (context) {
      context = null;
      throw Error("Try to set '".concat(key, "' of '").concat(stringifyElement(target), "' in get call"));
    }

    var entry = getEntry(target, key);
    var newValue = setter(target, value, entry.value);

    if (newValue !== entry.value) {
      entry.state += 1;
      entry.value = newValue;
      dispatchDeep(entry);
    }
  }
  function invalidate(target, key, clearValue) {
    if (context) {
      context = null;
      throw Error("Try to invalidate '".concat(key, "' in '").concat(stringifyElement(target), "' get call"));
    }

    var entry = getEntry(target, key);
    entry.checksum = 0;
    dispatchDeep(entry);

    if (clearValue) {
      entry.value = undefined;
    }
  }
  function observe(target, key, fn) {
    var entry = getEntry(target, key);
    entry.observed = true;
    return subscribe(entry, fn);
  }

  function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

  function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

  function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

  function _possibleConstructorReturn(self, call) { if (call && (_typeof$2(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

  function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

  function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

  function _wrapNativeSuper(Class) { var _cache = typeof Map === "function" ? new Map() : undefined; _wrapNativeSuper = function _wrapNativeSuper(Class) { if (Class === null || !_isNativeFunction(Class)) return Class; if (typeof Class !== "function") { throw new TypeError("Super expression must either be null or a function"); } if (typeof _cache !== "undefined") { if (_cache.has(Class)) return _cache.get(Class); _cache.set(Class, Wrapper); } function Wrapper() { return _construct(Class, arguments, _getPrototypeOf(this).constructor); } Wrapper.prototype = Object.create(Class.prototype, { constructor: { value: Wrapper, enumerable: false, writable: true, configurable: true } }); return _setPrototypeOf(Wrapper, Class); }; return _wrapNativeSuper(Class); }

  function isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

  function _construct(Parent, args, Class) { if (isNativeReflectConstruct()) { _construct = Reflect.construct; } else { _construct = function _construct(Parent, args, Class) { var a = [null]; a.push.apply(a, args); var Constructor = Function.bind.apply(Parent, a); var instance = new Constructor(); if (Class) _setPrototypeOf(instance, Class.prototype); return instance; }; } return _construct.apply(null, arguments); }

  function _isNativeFunction(fn) { return Function.toString.call(fn).indexOf("[native code]") !== -1; }

  function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

  function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

  function _typeof$2(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$2 = function _typeof(obj) { return typeof obj; }; } else { _typeof$2 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$2(obj); }
  /* istanbul ignore next */

  try {
    process.env.NODE_ENV;
  } catch (e) {
    var process = {
      env: {
        NODE_ENV: 'production'
      }
    };
  } // eslint-disable-line


  var defaultMethod = function defaultMethod(host, value) {
    return value;
  };

  function compile(Hybrid, descriptors) {
    Hybrid.hybrids = descriptors;
    Hybrid.callbacks = [];
    Object.keys(descriptors).forEach(function (key) {
      var desc = descriptors[key];

      var type = _typeof$2(desc);

      var config;

      if (type === 'function') {
        config = key === 'render' ? render(desc) : {
          get: desc
        };
      } else if (type !== 'object' || desc === null || Array.isArray(desc)) {
        config = property(desc);
      } else {
        config = {
          get: desc.get || defaultMethod,
          set: desc.set || !desc.get && defaultMethod || undefined,
          connect: desc.connect,
          observe: desc.observe
        };
      }

      Object.defineProperty(Hybrid.prototype, key, {
        get: function get$1() {
          return get(this, key, config.get);
        },
        set: config.set && function set$1(newValue) {
          set(this, key, config.set, newValue);
        },
        enumerable: true,
        configurable: process.env.NODE_ENV !== 'production'
      });

      if (config.connect) {
        Hybrid.callbacks.push(function (host) {
          return config.connect(host, key, function () {
            invalidate(host, key);
          });
        });
      }

      if (config.observe) {
        Hybrid.callbacks.push(function (host) {
          var lastValue;
          return observe(host, key, function () {
            var value = host[key];

            if (value !== lastValue) {
              config.observe(host, value, lastValue);
              lastValue = value;
            }
          });
        });
      }
    });
  }

  var disconnects = new WeakMap();

  function defineElement(tagName, hybridsOrConstructor) {
    var type = _typeof$2(hybridsOrConstructor);

    if (type !== 'object' && type !== 'function') {
      throw TypeError("Second argument must be an object or a function: ".concat(type));
    }

    var CustomElement = window.customElements.get(tagName);

    if (type === 'function') {
      if (CustomElement !== hybridsOrConstructor) {
        return window.customElements.define(tagName, hybridsOrConstructor);
      }

      return CustomElement;
    }

    if (CustomElement) {
      if (CustomElement.hybrids === hybridsOrConstructor) {
        return CustomElement;
      }

      throw Error("Element '".concat(tagName, "' already defined"));
    }

    var Hybrid =
    /*#__PURE__*/
    function (_HTMLElement) {
      _inherits(Hybrid, _HTMLElement);

      function Hybrid() {
        _classCallCheck(this, Hybrid);

        return _possibleConstructorReturn(this, _getPrototypeOf(Hybrid).apply(this, arguments));
      }

      _createClass(Hybrid, [{
        key: "connectedCallback",
        value: function connectedCallback() {
          var callbacks = this.constructor.callbacks;
          var list = [];

          for (var index = 0; index < callbacks.length; index += 1) {
            var cb = callbacks[index](this);
            if (cb) list.push(cb);
          }

          disconnects.set(this, list);
        }
      }, {
        key: "disconnectedCallback",
        value: function disconnectedCallback() {
          var list = disconnects.get(this);

          for (var index = 0; index < list.length; index += 1) {
            list[index]();
          }
        }
      }], [{
        key: "name",
        get: function get() {
          return tagName;
        }
      }]);

      return Hybrid;
    }(_wrapNativeSuper(HTMLElement));

    compile(Hybrid, hybridsOrConstructor);
    customElements.define(tagName, Hybrid);
    return Hybrid;
  }

  function defineMap(elements) {
    return Object.keys(elements).reduce(function (acc, key) {
      var tagName = pascalToDash(key);
      acc[key] = defineElement(tagName, elements[key]);
      return acc;
    }, {});
  }

  function define() {
    if (_typeof$2(arguments.length <= 0 ? undefined : arguments[0]) === 'object') {
      return defineMap(arguments.length <= 0 ? undefined : arguments[0]);
    }

    return defineElement.apply(void 0, arguments);
  }

  var map = new WeakMap();
  var dataMap = {
    get: function get(key, defaultValue) {
      var value = map.get(key);
      if (value) return value;

      if (defaultValue) {
        map.set(key, defaultValue);
      }

      return defaultValue;
    },
    set: function set(key, value) {
      map.set(key, value);
      return value;
    }
  };
  function getTemplateEnd(node) {
    var data; // eslint-disable-next-line no-cond-assign

    while (node && (data = dataMap.get(node)) && data.endNode) {
      node = data.endNode;
    }

    return node;
  }
  function removeTemplate(target) {
    if (target.nodeType !== Node.TEXT_NODE) {
      var child = target.childNodes[0];

      while (child) {
        target.removeChild(child);
        child = target.childNodes[0];
      }
    } else {
      var data = dataMap.get(target);

      if (data.startNode) {
        var endNode = getTemplateEnd(data.endNode);
        var node = data.startNode;
        var lastNextSibling = endNode.nextSibling;

        while (node) {
          var nextSibling = node.nextSibling;
          node.parentNode.removeChild(node);
          node = nextSibling !== lastNextSibling && nextSibling;
        }
      }
    }
  }

  var arrayMap = new WeakMap();

  function movePlaceholder(target, previousSibling) {
    var data = dataMap.get(target);
    var startNode = data.startNode;
    var endNode = getTemplateEnd(data.endNode);
    previousSibling.parentNode.insertBefore(target, previousSibling.nextSibling);
    var prevNode = target;
    var node = startNode;

    while (node) {
      var nextNode = node.nextSibling;
      prevNode.parentNode.insertBefore(node, prevNode.nextSibling);
      prevNode = node;
      node = nextNode !== endNode.nextSibling && nextNode;
    }
  }

  function resolveArray(host, target, value) {
    var lastEntries = arrayMap.get(target);
    var entries = value.map(function (item, index) {
      return {
        id: Object.prototype.hasOwnProperty.call(item, 'id') ? item.id : index,
        value: item,
        placeholder: null,
        available: true
      };
    });
    arrayMap.set(target, entries);

    if (lastEntries) {
      var ids = new Set();
      entries.forEach(function (entry) {
        return ids.add(entry.id);
      });
      lastEntries = lastEntries.filter(function (entry) {
        if (!ids.has(entry.id)) {
          removeTemplate(entry.placeholder);
          entry.placeholder.parentNode.removeChild(entry.placeholder);
          return false;
        }

        return true;
      });
    }

    var previousSibling = target;
    var lastIndex = value.length - 1;
    var data = dataMap.get(target);

    for (var index = 0; index < entries.length; index += 1) {
      var entry = entries[index];
      var matchedEntry = void 0;

      if (lastEntries) {
        for (var i = 0; i < lastEntries.length; i += 1) {
          if (lastEntries[i].available && lastEntries[i].id === entry.id) {
            matchedEntry = lastEntries[i];
            break;
          }
        }
      }

      var placeholder = void 0;

      if (matchedEntry) {
        matchedEntry.available = false;
        placeholder = matchedEntry.placeholder;

        if (placeholder.previousSibling !== previousSibling) {
          movePlaceholder(placeholder, previousSibling);
        }

        if (matchedEntry.value !== entry.value) {
          resolveValue(host, placeholder, entry.value);
        }
      } else {
        placeholder = document.createTextNode('');
        previousSibling.parentNode.insertBefore(placeholder, previousSibling.nextSibling);
        resolveValue(host, placeholder, entry.value);
      }

      previousSibling = getTemplateEnd(dataMap.get(placeholder).endNode || placeholder);
      if (index === 0) data.startNode = placeholder;
      if (index === lastIndex) data.endNode = previousSibling;
      entry.placeholder = placeholder;
    }

    if (lastEntries) {
      lastEntries.forEach(function (entry) {
        if (entry.available) {
          removeTemplate(entry.placeholder);
          entry.placeholder.parentNode.removeChild(entry.placeholder);
        }
      });
    }
  }

  function _typeof$3(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$3 = function _typeof(obj) { return typeof obj; }; } else { _typeof$3 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$3(obj); }
  function resolveValue(host, target, value) {
    var type = Array.isArray(value) ? 'array' : _typeof$3(value);
    var data = dataMap.get(target, {});

    if (data.type !== type) {
      removeTemplate(target);
      if (type === 'array') arrayMap.delete(target);
      data = dataMap.set(target, {
        type: type
      });

      if (target.textContent !== '') {
        target.textContent = '';
      }
    }

    switch (type) {
      case 'function':
        value(host, target);
        break;

      case 'array':
        resolveArray(host, target, value);
        break;

      default:
        target.textContent = type === 'number' || value ? value : '';
    }
  }

  function _typeof$4(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$4 = function _typeof(obj) { return typeof obj; }; } else { _typeof$4 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$4(obj); }

  var eventMap = new WeakMap();
  function resolveEventListener(eventType) {
    return function (host, target, value, lastValue) {
      if (lastValue) {
        target.removeEventListener(eventType, eventMap.get(lastValue), lastValue.options !== undefined ? lastValue.options : false);
      }

      if (value) {
        if (typeof value !== 'function') {
          throw Error("Event listener must be a function: ".concat(_typeof$4(value)));
        }

        eventMap.set(value, value.bind(null, host));
        target.addEventListener(eventType, eventMap.get(value), value.options !== undefined ? value.options : false);
      }
    };
  }

  function _typeof$5(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$5 = function _typeof(obj) { return typeof obj; }; } else { _typeof$5 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$5(obj); }

  function normalizeValue(value) {
    var set = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new Set();

    if (Array.isArray(value)) {
      value.forEach(function (className) {
        return set.add(className);
      });
    } else if (value !== null && _typeof$5(value) === 'object') {
      Object.keys(value).forEach(function (key) {
        return value[key] && set.add(key);
      });
    } else {
      set.add(value);
    }

    return set;
  }

  var classMap = new WeakMap();
  function resolveClassList(host, target, value) {
    var previousList = classMap.get(target) || new Set();
    var list = normalizeValue(value);
    classMap.set(target, list);
    list.forEach(function (className) {
      target.classList.add(className);
      previousList.delete(className);
    });
    previousList.forEach(function (className) {
      target.classList.remove(className);
    });
  }

  function _typeof$6(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$6 = function _typeof(obj) { return typeof obj; }; } else { _typeof$6 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$6(obj); }
  var styleMap = new WeakMap();
  function resolveStyle(host, target, value) {
    if (value === null || _typeof$6(value) !== 'object') {
      throw TypeError("Style value must be an object in ".concat(stringifyElement(target), ":"), value);
    }

    var previousMap = styleMap.get(target) || new Map();
    var nextMap = Object.keys(value).reduce(function (map, key) {
      var dashKey = camelToDash(key);
      var styleValue = value[key];

      if (!styleValue && styleValue !== 0) {
        target.style.removeProperty(dashKey);
      } else {
        target.style.setProperty(dashKey, styleValue);
      }

      map.set(dashKey, styleValue);
      previousMap.delete(dashKey);
      return map;
    }, new Map());
    previousMap.forEach(function (styleValue, key) {
      target.style[key] = '';
    });
    styleMap.set(target, nextMap);
  }

  function resolveProperty(attrName, propertyName, isSVG) {
    if (propertyName.substr(0, 2) === 'on') {
      var eventType = propertyName.substr(2);
      return resolveEventListener(eventType);
    }

    switch (attrName) {
      case 'class':
        return resolveClassList;

      case 'style':
        return resolveStyle;

      default:
        return function (host, target, value) {
          if (!isSVG && !(target instanceof SVGElement) && propertyName in target) {
            if (target[propertyName] !== value) {
              target[propertyName] = value;
            }
          } else if (value === false || value === undefined || value === null) {
            target.removeAttribute(attrName);
          } else {
            var attrValue = value === true ? '' : String(value);
            target.setAttribute(attrName, attrValue);
          }
        };
    }
  }

  function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

  function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

  function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

  function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

  function _typeof$7(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$7 = function _typeof(obj) { return typeof obj; }; } else { _typeof$7 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$7(obj); }
  /* istanbul ignore next */

  try {
    process$1.env.NODE_ENV;
  } catch (e) {
    var process$1 = {
      env: {
        NODE_ENV: 'production'
      }
    };
  } // eslint-disable-line


  var TIMESTAMP = Date.now();
  var getPlaceholder = function getPlaceholder() {
    var id = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    return "{{h-".concat(TIMESTAMP, "-").concat(id, "}}");
  };
  var PLACEHOLDER_REGEXP_TEXT = getPlaceholder('(\\d+)');
  var PLACEHOLDER_REGEXP_EQUAL = new RegExp("^".concat(PLACEHOLDER_REGEXP_TEXT, "$"));
  var PLACEHOLDER_REGEXP_ALL = new RegExp(PLACEHOLDER_REGEXP_TEXT, 'g');
  var ATTR_PREFIX = "--".concat(TIMESTAMP, "--");
  var ATTR_REGEXP = new RegExp(ATTR_PREFIX, 'g');
  var preparedTemplates = new WeakMap();
  /* istanbul ignore next */

  function applyShadyCSS(template, tagName) {
    if (!tagName) return template;
    return shadyCSS(function (shady) {
      var map = preparedTemplates.get(template);

      if (!map) {
        map = new Map();
        preparedTemplates.set(template, map);
      }

      var clone = map.get(tagName);

      if (!clone) {
        clone = document.createElement('template');
        clone.content.appendChild(template.content.cloneNode(true));
        map.set(tagName, clone);
        var styles = clone.content.querySelectorAll('style');
        Array.from(styles).forEach(function (style) {
          var count = style.childNodes.length + 1;

          for (var i = 0; i < count; i += 1) {
            style.parentNode.insertBefore(document.createTextNode(getPlaceholder()), style);
          }
        });
        shady.prepareTemplate(clone, tagName.toLowerCase());
      }

      return clone;
    }, template);
  }

  function createSignature(parts, styles) {
    var signature = parts.reduce(function (acc, part, index) {
      if (index === 0) {
        return part;
      }

      if (parts.slice(index).join('').match(/^\s*<\/\s*(table|tr|thead|tbody|tfoot|colgroup)>/)) {
        return "".concat(acc, "<!--").concat(getPlaceholder(index - 1), "-->").concat(part);
      }

      return acc + getPlaceholder(index - 1) + part;
    }, '');

    if (styles) {
      signature += "<style>\n".concat(styles.join('\n/*------*/\n'), "\n</style>");
    }
    /* istanbul ignore if */


    if (IS_IE) {
      return signature.replace(/style\s*=\s*(["][^"]+["]|['][^']+[']|[^\s"'<>/]+)/g, function (match) {
        return "".concat(ATTR_PREFIX).concat(match);
      });
    }

    return signature;
  }

  function getPropertyName(string) {
    return string.replace(/\s*=\s*['"]*$/g, '').split(' ').pop();
  }

  function replaceComments(fragment) {
    var iterator = document.createNodeIterator(fragment, NodeFilter.SHOW_COMMENT, null, false);
    var node; // eslint-disable-next-line no-cond-assign

    while (node = iterator.nextNode()) {
      if (PLACEHOLDER_REGEXP_EQUAL.test(node.textContent)) {
        node.parentNode.insertBefore(document.createTextNode(node.textContent), node);
        node.parentNode.removeChild(node);
      }
    }
  }

  function createInternalWalker(context) {
    var node;
    return {
      get currentNode() {
        return node;
      },

      nextNode: function nextNode() {
        if (node === undefined) {
          node = context.childNodes[0];
        } else if (node.childNodes.length) {
          node = node.childNodes[0];
        } else if (node.nextSibling) {
          node = node.nextSibling;
        } else {
          node = node.parentNode.nextSibling;
        }

        return !!node;
      }
    };
  }

  function createExternalWalker(context) {
    return document.createTreeWalker(context, // eslint-disable-next-line no-bitwise
    NodeFilter.SHOW_ELEMENT | NodeFilter.SHOW_TEXT, null, false);
  }
  /* istanbul ignore next */


  var createWalker = _typeof$7(window.ShadyDOM) === 'object' && window.ShadyDOM.inUse ? createInternalWalker : createExternalWalker;
  var container = document.createElement('div');
  function compileTemplate(rawParts, isSVG, styles) {
    var template = document.createElement('template');
    var parts = [];
    var signature = createSignature(rawParts, styles);
    if (isSVG) signature = "<svg>".concat(signature, "</svg>");
    /* istanbul ignore if */

    if (IS_IE) {
      template.innerHTML = signature;
    } else {
      container.innerHTML = "<template>".concat(signature, "</template>");
      template.content.appendChild(container.children[0].content);
    }

    if (isSVG) {
      var svgRoot = template.content.firstChild;
      template.content.removeChild(svgRoot);
      Array.from(svgRoot.childNodes).forEach(function (node) {
        return template.content.appendChild(node);
      });
    }

    replaceComments(template.content);
    var compileWalker = createWalker(template.content);
    var compileIndex = 0;

    var _loop = function _loop() {
      var node = compileWalker.currentNode;

      if (node.nodeType === Node.TEXT_NODE) {
        var text = node.textContent;

        if (!text.match(PLACEHOLDER_REGEXP_EQUAL)) {
          var results = text.match(PLACEHOLDER_REGEXP_ALL);

          if (results) {
            var currentNode = node;
            results.reduce(function (acc, placeholder) {
              var _acc$pop$split = acc.pop().split(placeholder),
                  _acc$pop$split2 = _slicedToArray(_acc$pop$split, 2),
                  before = _acc$pop$split2[0],
                  next = _acc$pop$split2[1];

              if (before) acc.push(before);
              acc.push(placeholder);
              if (next) acc.push(next);
              return acc;
            }, [text]).forEach(function (part, index) {
              if (index === 0) {
                currentNode.textContent = part;
              } else {
                currentNode = currentNode.parentNode.insertBefore(document.createTextNode(part), currentNode.nextSibling);
              }
            });
          }
        }

        var equal = node.textContent.match(PLACEHOLDER_REGEXP_EQUAL);

        if (equal) {
          /* istanbul ignore else */
          if (!IS_IE) node.textContent = '';
          parts[equal[1]] = [compileIndex, resolveValue];
        }
      } else {
        /* istanbul ignore else */
        // eslint-disable-next-line no-lonely-if
        if (node.nodeType === Node.ELEMENT_NODE) {
          Array.from(node.attributes).forEach(function (attr) {
            var value = attr.value.trim();
            /* istanbul ignore next */

            var name = IS_IE ? attr.name.replace(ATTR_PREFIX, '') : attr.name;
            var equal = value.match(PLACEHOLDER_REGEXP_EQUAL);

            if (equal) {
              var propertyName = getPropertyName(rawParts[equal[1]]);
              parts[equal[1]] = [compileIndex, resolveProperty(name, propertyName, isSVG)];
              node.removeAttribute(attr.name);
            } else {
              var _results = value.match(PLACEHOLDER_REGEXP_ALL);

              if (_results) {
                var partialName = "attr__".concat(name);

                _results.forEach(function (placeholder, index) {
                  var _placeholder$match = placeholder.match(PLACEHOLDER_REGEXP_EQUAL),
                      _placeholder$match2 = _slicedToArray(_placeholder$match, 2),
                      id = _placeholder$match2[1];

                  parts[id] = [compileIndex, function (host, target, attrValue) {
                    var data = dataMap.get(target, {});
                    data[partialName] = (data[partialName] || value).replace(placeholder, attrValue == null ? '' : attrValue);

                    if (_results.length === 1 || index + 1 === _results.length) {
                      target.setAttribute(name, data[partialName]);
                      data[partialName] = undefined;
                    }
                  }];
                });

                attr.value = '';
                /* istanbul ignore next */

                if (IS_IE && name !== attr.name) {
                  node.removeAttribute(attr.name);
                  node.setAttribute(name, '');
                }
              }
            }
          });
        }
      }

      compileIndex += 1;
    };

    while (compileWalker.nextNode()) {
      _loop();
    }

    return function updateTemplateInstance(host, target, args) {
      var data = dataMap.get(target, {
        type: 'function'
      });

      if (template !== data.template) {
        if (data.template || target.nodeType === Node.ELEMENT_NODE) removeTemplate(target);
        data.lastArgs = null;
        var fragment = document.importNode(applyShadyCSS(template, host.tagName).content, true);
        var renderWalker = createWalker(fragment);
        var clonedParts = parts.slice(0);
        var renderIndex = 0;
        var currentPart = clonedParts.shift();
        var markers = [];
        data.template = template;
        data.markers = markers;

        while (renderWalker.nextNode()) {
          var node = renderWalker.currentNode;

          if (node.nodeType === Node.TEXT_NODE) {
            /* istanbul ignore next */
            if (PLACEHOLDER_REGEXP_EQUAL.test(node.textContent)) {
              node.textContent = '';
            } else if (IS_IE) {
              node.textContent = node.textContent.replace(ATTR_REGEXP, '');
            }
          }

          while (currentPart && currentPart[0] === renderIndex) {
            markers.push([node, currentPart[1]]);
            currentPart = clonedParts.shift();
          }

          renderIndex += 1;
        }

        if (target.nodeType === Node.TEXT_NODE) {
          data.startNode = fragment.childNodes[0];
          data.endNode = fragment.childNodes[fragment.childNodes.length - 1];
          var previousChild = target;
          var child = fragment.childNodes[0];

          while (child) {
            target.parentNode.insertBefore(child, previousChild.nextSibling);
            previousChild = child;
            child = fragment.childNodes[0];
          }
        } else {
          target.appendChild(fragment);
        }
      }

      for (var index = 0; index < data.markers.length; index += 1) {
        var _data$markers$index = _slicedToArray(data.markers[index], 2),
            _node = _data$markers$index[0],
            marker = _data$markers$index[1];

        if (!data.lastArgs || data.lastArgs[index] !== args[index]) {
          marker(host, _node, args[index], data.lastArgs ? data.lastArgs[index] : undefined);
        }
      }

      if (target.nodeType !== Node.TEXT_NODE) {
        shadyCSS(function (shady) {
          if (host.shadowRoot) {
            if (data.lastArgs) {
              shady.styleSubtree(host);
            } else {
              shady.styleElement(host);
            }
          }
        });
      }

      data.lastArgs = args;
    };
  }

  var setCache = new Map();
  function set$1(propertyName, value) {
    if (!propertyName) throw Error("Target property name missing: ".concat(propertyName));

    if (arguments.length === 2) {
      return function (host) {
        host[propertyName] = value;
      };
    }

    var fn = setCache.get(propertyName);

    if (!fn) {
      fn = function fn(host, _ref) {
        var target = _ref.target;
        host[propertyName] = target.value;
      };

      setCache.set(propertyName, fn);
    }

    return fn;
  }
  var promiseMap = new WeakMap();
  function resolve(promise, placeholder) {
    var delay = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 200;
    return function (host, target) {
      var timeout;

      if (placeholder) {
        timeout = setTimeout(function () {
          timeout = undefined;
          requestAnimationFrame(function () {
            placeholder(host, target);
          });
        }, delay);
      }

      promiseMap.set(target, promise);
      promise.then(function (template) {
        if (timeout) clearTimeout(timeout);

        if (promiseMap.get(target) === promise) {
          template(host, target);
          promiseMap.set(target, null);
        }
      });
    };
  }

  var helpers = /*#__PURE__*/Object.freeze({
    set: set$1,
    resolve: resolve
  });

  var PLACEHOLDER = getPlaceholder();
  var SVG_PLACEHOLDER = getPlaceholder('svg');
  var templatesMap = new Map();
  var stylesMap = new WeakMap();
  var methods = {
    define: function define$1(elements) {
      define(elements);
      return this;
    },
    key: function key(id) {
      this.id = id;
      return this;
    },
    style: function style() {
      for (var _len = arguments.length, styles = new Array(_len), _key = 0; _key < _len; _key++) {
        styles[_key] = arguments[_key];
      }

      stylesMap.set(this, styles);
      return this;
    }
  };

  function create(parts, args, isSVG) {
    var createTemplate = function createTemplate(host) {
      var target = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : host;
      var styles = stylesMap.get(createTemplate);
      var id = parts.join(PLACEHOLDER);
      if (styles) id += styles.join(PLACEHOLDER);
      if (isSVG) id += SVG_PLACEHOLDER;
      var render = templatesMap.get(id);

      if (!render) {
        render = compileTemplate(parts, isSVG, styles);
        templatesMap.set(id, render);
      }

      render(host, target, args);
    };

    return Object.assign(createTemplate, methods);
  }

  function html(parts) {
    for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      args[_key2 - 1] = arguments[_key2];
    }

    return create(parts, args);
  }
  function svg(parts) {
    for (var _len3 = arguments.length, args = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
      args[_key3 - 1] = arguments[_key3];
    }

    return create(parts, args, true);
  }
  Object.assign(html, helpers);
  Object.assign(svg, helpers);

  var style = html`
	<style>

		svg .border {
			stroke: #353839;
			stroke-width: 5px;
			fill: none;
		}

		svg .axes line {
			fill: none;
			stroke: #eee;
			stroke-width: 1px;
		}

		svg .ticks {
			fill: none;
			stroke: #353839;
			stroke-width: 2px;
		}

		svg text {
			font-family: monospace;
			font-family: var(--font-family);
			fill: #353839;
		}

		svg .labels text {
			font-size: 10px;
			text-anchor: middle;
		}

		svg .titles text {
			font-size: 12px;
			font-size: var(--font-size);
			font-weight: bold;
			text-anchor: middle;
		}

		svg .data circle {
			fill: #353839;
		}

	</style>
`;

  const margin = {
  	get: () => ({top: 10, right: 10, bottom: 10, left: 10})
  };

  const width = {
  	get: ({side, margin}) => side - margin.left - margin.right
  };

  const height = {
  	get: ({side, margin}) => side - margin.top - margin.bottom
  };

  const center = {
  	get: ({width, height}) => [width / 2, height / 2]
  };

  const globalTransform = {
  	get: ({margin, radius}) => {
  		// Eye-adjusted vertical shift
  		const shift = 0.8 * (radius * (1 - Math.sin(Math.PI / 6))) / 2;

  		return `translate(${margin.left}, ${margin.top + shift})`;
  	}
  };

  const borderPoints = {
  	get: ({vertices}) => `
		${vertices[0][0]},${vertices[0][1]}
		${vertices[1][0]},${vertices[1][1]}
		${vertices[2][0]},${vertices[2][1]}
	`
  };

  // Vertices are sorted as [right, bottom, left].
  const vertices = {
  	get: ({radius, center}) => [0, Math.PI * (2 / 3), Math.PI * (4 / 3)]
  		.map(angle => [
  			center[0] + (radius * Math.sin(angle)),
  			center[1] - (radius * Math.cos(angle))
  		])
  };

  const radius = {
  	get: ({width, height}) => {
  		const scaleFactor = 1 / 2;
  		return Math.min(width, height) * scaleFactor;
  	}
  };

  const step = {
  	get: ({radius}) => (radius * Math.sqrt(3)) / 10
  };

  const pattern = {
  	get: ({width, height, step, radius}) => svg`
		<pattern
			id="ternary-pattern"
			patternUnits="userSpaceOnUse"
			patternTransform="translate(
				${(width / 2) + (step / 2)}, ${(height / 2) - radius}
			)"
			width="${2 * step}"
			height="${Math.sqrt(3) * step}"
		>
			<rect
				fill="rgba(0, 0, 0, 0.1)"
				width="${2 * step}"
				height="${Math.sqrt(3) * step}">
			></rect>
			<polygon
				fill="rgba(0, 0, 0, 0.1)"
				points="
					0,${(Math.sqrt(3) / 2) * step}
					${2 * step},${(Math.sqrt(3) / 2) * step}
					${1.5 * step},0
					${Number(step)},${(Math.sqrt(3) / 2) * step}
					${0.5 * step},0"
			></polygon>
			<polygon
				fill="rgba(0, 0, 0, 0.1)"
				points="
					0,${Math.sqrt(3) * step}
					${2 * step},${Math.sqrt(3) * step}
					${2 * step}, ${(Math.sqrt(3) / 2) * step}
					${1.5 * step},${Math.sqrt(3) * step}
					${Number(step)},${(Math.sqrt(3) / 2) * step}
					${0.5 * step},${Math.sqrt(3) * step}
					0, ${(Math.sqrt(3) / 2) * step}"
			></polygon>
		</pattern>
	`
  };

  function renderCircles(pointsCoordinates) {
  	return pointsCoordinates.map(d => svg`
		<circle
			cx="${d[0]}"
			cy="${d[1]}"
			r=5
		></cirlce>
	`);
  }

  const pointsCoordinates = {
  	get: ({data, vertices}) => data.data.map(d => pointCoordinate(d, vertices))
  };

  function pointCoordinate(d, v) {
  	return [
  		(d.right * v[0][0]) + (d.bottom * v[1][0]) + (d.left * v[2][0]),
  		(d.right * v[0][1]) + (d.bottom * v[1][1]) + (d.left * v[2][1])
  	];
  }

  function getAngleFromTicksPositions(ticksPositions) {
  	const p = ticksPositions;
  	const dy = p.slice(-1)[0][1] - p[0][1];
  	const dx = p.slice(-1)[0][0] - p[0][0];
  	return getAngle(dx, dy);
  }

  // Get the angle of a segment of width <dx> and height <dy> with the left vertice in [0, 0].
  function getAngle(dx, dy) {
  	if (dx > 0 && dy >= 0) {
  		return Math.atan(dy / dx);
  	}

  	if (dx > 0 && dy < 0) {
  		return Math.atan(dy / dx) + (Math.PI * 2);
  	}

  	if (dx < 0) {
  		return Math.atan(dy / dx) + Math.PI;
  	}

  	if (dx === 0 && dy > 0) {
  		return Math.PI / 2;
  	}

  	if (dx === 0 && dy < 0) {
  		return Math.PI * (3 / 2);
  	}

  	// Default if both <dx> and <dy> are zero.
  	return 0;
  }

  function renderTicks(ticks) {
  	return ticks.map(d => svg`
		<line
			x1="${d[0][0]}"
			y1="${d[0][1]}"
			x2="${d[1][0]}"
			y2="${d[1][1]}"
		></line>
	`);
  }

  const bottomTicksPositions = {
  	get: ({vertices, numberOfTicks}) => {
  		const v1 = vertices[1];
  		const v2 = vertices[2];
  		return thicksPositions(v1, v2, numberOfTicks);
  	}
  };

  const leftTicksPositions = {
  	get: ({vertices, numberOfTicks}) => {
  		const v1 = vertices[0];
  		const v2 = vertices[1];
  		return thicksPositions(v1, v2, numberOfTicks);
  	}
  };

  const rightTicksPositions = {
  	get: ({vertices, numberOfTicks}) => {
  		const v1 = vertices[2];
  		const v2 = vertices[0];
  		return thicksPositions(v1, v2, numberOfTicks);
  	}
  };

  const bottomTicks = {
  	get: ({bottomTicksPositions, tickLenght}) =>
  		getTicks(bottomTicksPositions, tickLenght)
  };

  const rightTicks = {
  	get: ({rightTicksPositions, tickLenght}) =>
  		getTicks(rightTicksPositions, tickLenght)
  };

  const leftTicks = {
  	get: ({leftTicksPositions, tickLenght}) =>
  		getTicks(leftTicksPositions, tickLenght)
  };

  function thicksPositions(v1, v2, N) {
  	const v = [v2[0] - v1[0], v2[1] - v1[1]];
  	return Array
  		.from({length: N}, (_, i) => [
  			v1[0] + ((v[0]) * (i / N)),
  			v1[1] + ((v[1]) * (i / N))
  		])
  		.slice(1);
  }

  function getTicks(ticksPositions, l) {
  	const a = getAngleFromTicksPositions(ticksPositions);
  	const ticksExternalVerticePositions = ticksPositions
  		.map(d => [d[0] + (l * Math.sin(a)), d[1] - (l * Math.cos(a))]);
  	return ticksPositions
  		.map((_, i) => [
  			ticksPositions[i],
  			ticksExternalVerticePositions[i]
  		]);
  }

  function renderLabels(labels) {
  	return labels.map(d => svg`
		<text
			x="${d.x}"
			y="${d.y}"
			dy=3
		>${d.text}</text>
	`);
  }

  const bottomLabels = {
  	get: ({bottomTicksPositions, tickLenght}) =>
  		getLabels(bottomTicksPositions, tickLenght * 2)
  };

  const rightLabels = {
  	get: ({rightTicksPositions, tickLenght}) =>
  		getLabels(rightTicksPositions, tickLenght * 2)
  };

  const leftLabels = {
  	get: ({leftTicksPositions, tickLenght}) =>
  		getLabels(leftTicksPositions, tickLenght * 2)
  };

  function getLabels(ticksPositions, l) {
  	const a = getAngleFromTicksPositions(ticksPositions);
  	const ticksExternalVerticePositions = ticksPositions
  		.map(d => [d[0] + (l * Math.sin(a)), d[1] - (l * Math.cos(a))])
  		.reverse(); // Set the right labels order.
  	return ticksPositions
  		.map((_, i) => ({
  			x: ticksExternalVerticePositions[i][0],
  			y: ticksExternalVerticePositions[i][1],
  			text: `${(i * 10) + 10}`
  		}));
  }

  const transformBottomTitle = {
  	get: ({vertices}) => `translate(
		${(vertices[1][0] + vertices[2][0]) / 2},
		${(vertices[1][1] + vertices[2][1]) / 2}
	)`
  };

  const bottomTitle = {
  	get: ({data}) => `${data.titles.bottom}`
  };

  const transformRightTitle = {
  	get: ({vertices}) => `translate(
		${(vertices[0][0] + vertices[1][0]) / 2},
		${(vertices[0][1] + vertices[1][1]) / 2}
	) rotate(60)`
  };

  const rightTitle = {
  	get: ({data}) => `${data.titles.right}`
  };

  const transformLeftTitle = {
  	get: ({vertices}) => `translate(
		${(vertices[2][0] + vertices[0][0]) / 2},
		${(vertices[2][1] + vertices[0][1]) / 2}
	) rotate(-60)`
  };

  const leftTitle = {
  	get: ({data}) => `${data.titles.left}`
  };

  function renderAxes(axes) {
  	return axes.map(d => svg`
		<line
			x1="${d[0][0]}"
			y1="${d[0][1]}"
			x2="${d[1][0]}"
			y2="${d[1][1]}"
		></line>
	`);
  }

  const bottomAxes = {
  	get: ({leftTicksPositions, rightTicksPositions}) => {
  		const p = leftTicksPositions;
  		const q = [...rightTicksPositions].reverse();
  		return p.map((_, i) => [p[i], q[i]]);
  	}
  };

  const leftAxes = {
  	get: ({bottomTicksPositions, rightTicksPositions}) => {
  		const p = bottomTicksPositions;
  		const q = [...rightTicksPositions].reverse();
  		return p.map((_, i) => [p[i], q[i]]);
  	}
  };

  const rightAxes = {
  	get: ({leftTicksPositions, bottomTicksPositions}) => {
  		const p = leftTicksPositions;
  		const q = [...bottomTicksPositions].reverse();
  		return p.map((_, i) => [p[i], q[i]]);
  	}
  };

  function render$1({
  	globalTransform,
  	borderPoints,
  	bottomAxes,
  	rightAxes,
  	leftAxes,
  	bottomTicks,
  	rightTicks,
  	leftTicks,
  	bottomLabels,
  	rightLabels,
  	leftLabels,
  	transformBottomTitle,
  	bottomTitle,
  	transformRightTitle,
  	rightTitle,
  	transformLeftTitle,
  	leftTitle,
  	pointsCoordinates,
  	pattern,
  	side
  }) {
  	return html`
		${style}
		<svg width=${side} height=${side}>
			<defs>${pattern}</defs>
			<g transform="${globalTransform}">
				<g class="ticks">
					<g class="bottom-ticks">${renderTicks(bottomTicks)}</g>
					<g class="bottom-ticks">${renderTicks(rightTicks)}</g>
					<g class="bottom-ticks">${renderTicks(leftTicks)}</g>
				</g>
				<g class="labels">
					<g class="bottom-labels">${renderLabels(bottomLabels)}</g>
					<g class="bottom-labels">${renderLabels(rightLabels)}</g>
					<g class="bottom-labels">${renderLabels(leftLabels)}</g>
				</g>
				<g class="titles">
					<g class="bottom-title" transform=${transformBottomTitle}>
						<text dx="10" dy="45">${bottomTitle}</text>
					</g>
					<g class="right-title" transform=${transformRightTitle}>
						<text dx="-10" dy="-40">${rightTitle}</text>
					</g>
					<g class="left-title" transform=${transformLeftTitle}>
						<text dx="-10" dy="-40">${leftTitle}</text>
					</g>
				</g>
				<g class="axes">
					<g class="bottom-axes">${renderAxes(bottomAxes)}</g>
					<g class="right-axes">${renderAxes(rightAxes)}</g>
					<g class="left-axes">${renderAxes(leftAxes)}</g>
				</g>
				<polygon
					class="border"
					points="${borderPoints}"
					fill="url(#ternary-pattern)"
				/>
				<g class="data">${renderCircles(pointsCoordinates)}</g>
			</g>
		</svg>
	`;
  }

  const TernaryPlot = {
  	data: property({
  		titles: {
  			bottom: '',
  			right: '',
  			left: ''
  		},
  		data: []
  	}),
  	side: 400,
  	numberOfTicks: 10,
  	tickLenght: 10,
  	margin,
  	width,
  	height,
  	center,
  	radius,
  	globalTransform,
  	vertices,
  	borderPoints,
  	bottomTicksPositions,
  	rightTicksPositions,
  	leftTicksPositions,
  	bottomAxes,
  	rightAxes,
  	leftAxes,
  	bottomTicks,
  	rightTicks,
  	leftTicks,
  	bottomLabels,
  	rightLabels,
  	leftLabels,
  	bottomTitle,
  	transformBottomTitle,
  	rightTitle,
  	transformRightTitle,
  	leftTitle,
  	transformLeftTitle,
  	pointsCoordinates,
  	pattern,
  	step,
  	render: render$1
  };

  define('ternary-plot', TernaryPlot);

  exports.TernaryPlot = TernaryPlot;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
